#========================================================================================
# package      DE_EPAGES::Content::API::Attributes::Shop
# state        private
# description  Implements the attributes Shop.Categories
#========================================================================================
package DE_EPAGES::Content::API::Attributes::Shop;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use feature 'switch';
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Content::API::ShortUrl qw (GetShortUrlByClass);

sub getAttributes {
    shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my %Values;
    foreach my $Name (@$aNames) {
        given($Name) {
            when (['ImprintID', 'TermsAndConditionsID', 'PrivacyPolicyID',
                'CustomerInformationID', 'ShippingAndPaymentInformationID']) {
                my $Folder;
                if ($Object->get('IsUnityRegistered') && $Object->existsChild('LegalPagesUnity')) {
                    # map the legal pages for Unity in case we have a Unity shops - UNITY-2678
                    $Folder = 'LegalPagesUnity';
                } else {
                    # normal ep6 shop or LegalPagesUnity folder not existing
                    $Folder = 'Categories';
                }
                my $AttributeName = $Name;
                # get attribute name without 'ID'
                $AttributeName =~ s/ID//;
                $Values{$Name} = $Object->child($Folder)->findChild($AttributeName);
            }
            when( 'CategoriesID' ) {
                $Values{$Name} = $Object->child('Categories')->id;
            }
            when( 'ContentExamples') {
                my $Parent = $Object->get('Categories');
                my @Categories = grep {$_->get('IsVisible')} @{$Parent->get('CategoryPages')};
                my @Examples;
                my $ProtocolAndServer = $Object->get('DomainName');

                if (scalar @Categories > 0) {
                    my $URI = GetShortUrlByClass($Parent, $Categories[0]->class, $Categories[0]->get('NameOrAlias', $LanguageID));
                    push @Examples, {'Example' => $ProtocolAndServer.'/'.$URI} ;
                }
                if (scalar @Categories > 1) {
                    my $URI = GetShortUrlByClass($Parent, $Categories[1]->class, $Categories[1]->get('NameOrAlias', $LanguageID));
                    push @Examples, {'Example' => $ProtocolAndServer.'/'.$URI} ;
                }
                $Values{$Name} = \@Examples;
            }
            when( 'ContentObjects' ) {
                $Values{$Name} = LoadClassByAlias('ContentObject')->getObjects('BySiteID', [$Object->id]);
            }
            when( 'DisplayTitle' ) {
                $Values{$Name} = $Object->child('Categories')->get('DisplayTitle', $LanguageID);
            }
        }#given
    }

    return \%Values;
}

1;
