#==============================================================================
# package      DE_EPAGES::Content::UI::Object
# state        private
#==============================================================================
package DE_EPAGES::Content::UI::Object;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Content::API::ShortUrl qw ( CreateAllShortUrls );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError);

sub ErrorDefinition {
    return {
        'WrongParent' => 'Invalid parent id: #ObjectParentID. Expected: #ParentID',
    }
}

#========================================================================================
# function     View
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->View();
#----------------------------------------------------------------------------------------
# description  Override View to change Locale for current Request if defined
#========================================================================================
sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $OverrideLocale = $Servlet->params('OverrideLocale');
    if (defined $OverrideLocale) {
        $Servlet->setLocale($OverrideLocale, 0);
        $Servlet->params('ViewAction', 'View');
    }
    return $self->SUPER::View($Servlet);
}

#========================================================================================
# function     SaveShortUrls
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveShortUrls();
#----------------------------------------------------------------------------------------
# description  Sets short urls to all content objects.
#========================================================================================
sub SaveShortUrls {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    eval {
        CreateAllShortUrls($Shop);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'Form'  => 'Save',
            'Name'  => 'ShortUrl',
        });
    }
    $Shop->updatePageCache;
    return;
}

#========================================================================================
# function     CreateObjectSnippet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->CreateObjectSnippet();
#----------------------------------------------------------------------------------------
# description  ViewAction to add blog entries, forum posts etc. in the MBO preview mode.
#========================================================================================
sub CreateObjectSnippet {
}
#----------------------------------------------------------------------------------------
sub _getBatchTaskNamesForSetAttributeOnListedObjects { ## no critic (ProhibitUnusedPrivateSubroutines) # this overrides the method from base package
    my $self = shift;
    my $aSuperBatchTaskNames = $self->SUPER::_getBatchTaskNamesForSetAttributeOnListedObjects();
    return [@{$aSuperBatchTaskNames}, 'UpdateCategories'];
}
#----------------------------------------------------------------------------------------
sub _getBatchTaskNamesForDelete { ## no critic (ProhibitUnusedPrivateSubroutines) # this overrides the method from base package
    my $self = shift;
    my $aSuperBatchTaskNames = $self->SUPER::_getBatchTaskNamesForDelete();
    return [@{$aSuperBatchTaskNames}, 'DeleteCategories'];
}

1;
