#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::AddressReceiver
# state        public
# description  Receiver Address element Adittion for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::AddressReceiver;

use base DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use strict;

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAddressReceiver = $self->assemble($CustomerAddress);
#----------------------------------------------------------------------------------------
# description  Create the hash with address data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $CustomerAddress | Address object | object
# return       $hAddressReceiver | DHL Versenden receiver address element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($CustomerAddress) = @_;
    
    my $hAddressNode = $self->_assembleAddress($CustomerAddress);
    my @Warnings = @{$hAddressNode->{'nodeWarnings'}};

    my $hAddressData = $CustomerAddress->get(
        [
            'Street2',
            'AddressExtension',
            'Company',
        ]
   );

    my $LeafNodeHelper = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper->new();

    if ($self->_isValidAddressAddition($hAddressData)) {
        $LeafNodeHelper->addLeaf('addressAddition', $self->_concatenateAddressAddition($hAddressData));

        $hAddressNode->{'nodeData'}{'Address'}{'addressAddition'} = $LeafNodeHelper->getLeaf('addressAddition');
    }

    if (isValid($hAddressData->{'Company'})) {
        $LeafNodeHelper->addLeaf('name2', $hAddressData->{'Company'});

        $hAddressNode->{'nodeData'}{'Address'}{'name2'} = $LeafNodeHelper->getLeaf('name2');
    }

    push(@Warnings, @{$LeafNodeHelper->getWarnings()});
    $hAddressNode->{'nodeWarnings'} = \@Warnings;

    return $hAddressNode;
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleAddress {
    my $self = shift;
    my ($CustomerAddress) = @_;

    return $self->SUPER::assemble($CustomerAddress); 
}

sub _isValidAddressAddition {
    my $self = shift;
    my ($hAddressData) = @_;

    return isValid($hAddressData->{'AddressExtension'}) || isValid($hAddressData->{'Street2'}); 
}

sub _concatenateAddressAddition {
    my $self = shift;
    my ($hAddressData) = @_;

    my @AddressAddition;

    if (isValid($hAddressData->{'AddressExtension'})) {
        push(@AddressAddition, $hAddressData->{'AddressExtension'});
    }

    if (isValid($hAddressData->{'Street2'})) {
        push(@AddressAddition, $hAddressData->{'Street2'}); 
    }

    return join(' ', @AddressAddition);
}

1;