#==============================================================================
# package      DE_EPAGES::Ebay::API::EbayErrors
# state        public
#------------------------------------------------------------------------------
# description  Ebay Error Code handling.
#==============================================================================
package DE_EPAGES::Ebay::API::EbayErrors;
use base Exporter;

use strict;
our @EXPORT_OK = qw (
    TokenOrUserErrors
    IgnoreWarnings
);

#========================================================================================
# function     TokenOrUserErrors
# state        public
#----------------------------------------------------------------------------------------
# syntax       TokenOrUserErrors( );
#----------------------------------------------------------------------------------------
# description  These Error codes trigger sending a TokenError mail to the shop owner.
#----------------------------------------------------------------------------------------
# return       $aErrorCodes | list of error codes | array.int
#========================================================================================
sub TokenOrUserErrors () {
    return (930,931,932,16110,16118,16119,17470,841);
}

#========================================================================================
# function     IgnoreWarnings
# state        public
#----------------------------------------------------------------------------------------
# syntax       IgnoreWarnings( );
#----------------------------------------------------------------------------------------
# description  We don't log these EbayError codes, because we know about them.
#               (Warnings from ebay regarding the API version and deprecated objects... )
#----------------------------------------------------------------------------------------
# return       $aErrorCodes | list of error codes | array.int
#========================================================================================
sub IgnoreWarnings () {
    return (21927,219450,219451,21916516,21916780,21916885,21917092);
}

#21927      Short error: Unrecognized element <replaceable_value> in request message. Long error: The element <replaceable_value> was found in the input request messa
#219450     Short error: Deprecated object
#219451     Short error: Object no longer supported.
#21916516   Short error: Category replaceable_value requires custom item specifics.
#21916780   Short error: Featured Plus is no longer available.
#21916885   Short error: Dropped condition from Item specifics.
#21917092   Short error: Requested Quantity revision is redundant.


1;
