#========================================================================================
# package      DE_EPAGES::Ebay::UI::Shop
# state        private
# base         DE_EPAGES::Presentation::UI::Object
#========================================================================================
package DE_EPAGES::Ebay::UI::Shop;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::PerlTools qw( AddDuration );
use DE_EPAGES::Ebay::API::Shop qw (AddEbayTemplateToShop);
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);

#==============================================================================
# Event handlers
#==============================================================================
sub SearchEbayOrders {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $SearchForm = $Servlet->form->form($Shop, 'SearchEbayOrders');

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'CreationDate');
        $Servlet->params('OrderDesc', 1);
    }

    #--- remove empty elements.
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;

    #--- do the search
    return $self->_ExecuteTransactionSearch($Servlet, $SearchForm);
}

sub ViewEbayOrders {
    my $self = shift;
    my ($Servlet) = @_;

    my %SearchForm = ();

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'CreationDate');
        $Servlet->params('OrderDesc', 1);
    }

    #--- do the search
    return $self->_ExecuteTransactionSearch($Servlet, \%SearchForm);
}

sub _ExecuteTransactionSearch {
    my $self = shift;
    my ($Servlet, $SearchForm) = @_;
    my $Shop = $Servlet->object;
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('EbayTransaction'), 'MBO-EbayOrders');
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Shop->id,
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter eq 'EndTimeTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }

    delete $SearchForm->{'Search'};
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub ApplyEbayTemplates {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    my $System = LoadRootObject();

    my $hValues = $Form->form($Shop, 'ListedObjects');
    my @ListObjects = map { $System->loadObject( $_->{'ListObjectID'}, 'EbayTemplate' ) }
                      @{$hValues->{'ListObjectIDs'}};

    my $LanguageCode2 = $Servlet->params('Language');
    AddEbayTemplateToShop($Shop, $_, $LanguageCode2) foreach @ListObjects;

    return;
}

sub ApplyEbayTemplate {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $LanguageCode2 = $Servlet->params('Language');

    my $EbaySystemTemplateID = $Servlet->form->value('EbaySystemTemplateID');

    AddEbayTemplateToShop($Shop, $Shop->loadObject($EbaySystemTemplateID, 'EbayTemplate'), $LanguageCode2);

    return;
}

1;
