#========================================================================================
# package      DE_EPAGES::Order::t::unit::OrderDocumentResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::Order::t::unit::OrderDocumentResource
#========================================================================================
package DE_EPAGES::Order::t::unit::OrderDocumentResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use Test::More;

use strict;

use constant EN_LOCALE => 'en_GB';
use constant EXPECTED_DEFAULT_PAGE_SIZE => 10;
use constant EXPECTED_DEFAULT_PAGE_NUMBER => 1;
use constant EXPECTED_COUNTS_RESULT => 1;

sub test {
    my $self = shift;

    $self->test_get();
    $self->test_get_with_two_documents();

    done_testing();
}

sub test_get {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DocumentsInsideOrder = 1;
    my $Mockery = $self->_createMocks($DocumentsInsideOrder);
    my $OrderDocumentResource = $Mockery->get('SUT');
    my $Order = $Mockery->get('CustomerOrder');

    my $hActualOrderDocumentsPage = $OrderDocumentResource->get($Order, EN_LOCALE);

    is($hActualOrderDocumentsPage->{'results'}, EXPECTED_COUNTS_RESULT, "$SubTestName - test expected number of order documents found");
    is($hActualOrderDocumentsPage->{'resultsPerPage'}, EXPECTED_DEFAULT_PAGE_SIZE, "$SubTestName - test expected results per page is default one");
    is($hActualOrderDocumentsPage->{'page'}, EXPECTED_DEFAULT_PAGE_NUMBER, "$SubTestName - test expected page number is default one");
    is_deeply($hActualOrderDocumentsPage->{'items'}, [$self->_createOrderDocumentData()], "$SubTestName - test expected items");
}

sub test_get_with_two_documents {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DocumentsInsideOrder = 2;
    my $Mockery = $self->_createMocks($DocumentsInsideOrder);
    my $OrderDocumentResource = $Mockery->get('SUT');
    my $Order = $Mockery->get('CustomerOrder');

    my $hActualOrderDocumentsPage = $OrderDocumentResource->get($Order, EN_LOCALE);

    is($hActualOrderDocumentsPage->{'results'}, $DocumentsInsideOrder, "$SubTestName - test number of order documents found");
}

#========================================================================================
# Mocks
#========================================================================================
sub _createMocks {
    my $self = shift;
    my ($OrderDocumentsCounter) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();
    $Mockery->register('SUT', ['OrderDocumentService', 'SearchDefinition', 'OrderDocuments', 'PagerSearch'], sub {
        my ($OrderDocumentService, $SearchDefinition, $OrderDocuments, $PagerSearch) = @_;

        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::Resource::OrderDocumentResource', []);
        $Mockify->addMockWithReturnValue('_createOrderDocumentService', $OrderDocumentService);
        $Mockify->addMockWithReturnValueAndParameterCheck('_GetSearchDefinition', $SearchDefinition, ['object', 'string']);
        $Mockify->addMockWithReturnValueAndParameterCheck('_LoadClassByAlias', $OrderDocuments->[0], [ {'string' => 'OrderDocument'} ]);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createPagerSearch', $PagerSearch, [ 'hashref' ]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('OrderDocumentService', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::OrderDocumentService', []);

        my $aOrderDocumentDataParameters = [ {'object' => 'DE_EPAGES::Order::API::Object::OrderDocument'}, {'string' => EN_LOCALE}];
        $Mockify->addMockWithReturnValueAndParameterCheck(
                'getOrderDocumentData', $self->_createOrderDocumentData(), $aOrderDocumentDataParameters
        );
        return $Mockify->getMockObject();
    });

    $Mockery->register('SearchDefinition', ['Shop'], sub {
        my ($Shop) = @_;
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Search::API::Object::SearchDefinition', {});
        return $MockifyEpagesObject->getMockObject();
    });

    $Mockery->register('OrderDocuments', [], sub {
        my @MockifiedObjects = ();
        my $MockifyEpagesObject1 = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::OrderDocument', {});
        push(@MockifiedObjects, $MockifyEpagesObject1->getMockObject());
        if ($OrderDocumentsCounter > 1) {
            my $MockifyEpagesObject2 = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::OrderDocument', {});
            push(@MockifiedObjects, $MockifyEpagesObject2->getMockObject());
        }
        return \@MockifiedObjects
    });

    $Mockery->register('PagerSearch', ['OrderDocuments'], sub {
        my ($aOrderDocuments) = @_;

        my $MockedEpagesObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Search::API::Object::PagerSearch', {});
        $MockedEpagesObject->mock('countItems', sub { return scalar(@$aOrderDocuments); });
        $MockedEpagesObject->mock('items', sub { return $aOrderDocuments; });
        return $MockedEpagesObject;
    });

    $Mockery->register('CustomerOrder', ['Shop'], sub {
        my ($Shop) = @_;
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {
            'Alias' => 'TestObject',
            'ID' => '1230',
            'Site' => $Shop,
        });
        return $MockifyEpagesObject->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'Alias' => 'TestShop',
            'LocaleID' => EN_LOCALE,
        });
        $MockifyEpagesObject->addMockWithReturnValueAndParameterCheck('existsSiteLocale', 1, [{'string' => EN_LOCALE}]);
        return $MockifyEpagesObject->getMockObject();
    });

    return $Mockery->assemble();
}

sub _createOrderDocumentData {
    return {
        'orderId' => '1234',
        'documentType' => 'Invoice',
        'documentNumber' => '1001',
        'creationDate' => '2017-10-16T09:43:10.000Z',
        'address' => {},
        'lineItemContainer' => {}
    }
}

__PACKAGE__->runScript();