#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Logger
# state        public
#----------------------------------------------------------------------------------------
# description  Logger class
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Logger;
use base qw ( Exporter );

use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::ThirdPartyConfig::API::Log qw (Log3rdParty);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use strict;

our @EXPORT_OK = qw (
    Filter
);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Logger
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     logPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       logPayment( $Description, $hStruture, $ReplaceValue );
#----------------------------------------------------------------------------------------
# description  Replace value in the logs (remove password)
#----------------------------------------------------------------------------------------
# input        $Description | Description | string
# input        $hStruture | hash, object or any structure you want to log | hashref
# input        $ReplaceValue | string to be replace at the log | string
#========================================================================================
sub logPayment {
    my $self = shift;
    my ($Description, $hStruture, $ReplaceValue) = @_;

    my $Filter;
    if(isValid($ReplaceValue)){
        $Filter = Filter($ReplaceValue);
    }
    $self->_LogPayment($Description, $hStruture, $Filter);
    return;
}
#========================================================================================
# function     logOnboarding
# state        public
#----------------------------------------------------------------------------------------
# syntax       logOnboarding( $DebugStuff, $aFilter );
#----------------------------------------------------------------------------------------
# description  logs to communication.log if LoggingOnboarding is actived.
#----------------------------------------------------------------------------------------
# input        $DebugStuff | data to be logged | ref.hash, ref.array, string, object
# input        $aFilter | (optional) filter (perl regular expression for
#                       s/$Filter[^\n]+\n/\n/gis) | ref.array.string
#========================================================================================
sub logOnboarding {
    my $self = shift;
    my ($DebugStuff, $aFilter) = @_;

    my $PaymentTypesFolder = LoadRootObject()->folder('PaymentTypes');
    Error('NoPaymentType', {'Alias' => 'PayPalPlus'}) unless $PaymentTypesFolder->existsChild('PayPalPlus');
    return unless $PaymentTypesFolder->child('PayPalPlus')->get('LoggingOnboarding');

    my $Section = (split(/::/, (caller(1))[3]))[-1];
    Log3rdParty('PayPalPlusOnboarding', $DebugStuff, $Section, $aFilter);
    return;
}
#========================================================================================
# function     Filter
# state        public
#----------------------------------------------------------------------------------------
# syntax       Filter( $ReplaceValue );
#----------------------------------------------------------------------------------------
# description  Function with a regular expression to replace a value in a string
#----------------------------------------------------------------------------------------
# input        $ReplaceValue | string to be replace at the log | string
# return       Function pointer | function pointer
#========================================================================================
sub Filter {
    my ($ReplaceValue) = @_; #TODO: AD-3112 check if we can use self here (make private)
    return sub {
        my ($hashString) = @_;
        ${$hashString} =~s/$ReplaceValue/---replaced---/msxig;
        return;
    }
}

sub _LogPayment {
    my $self = shift;
    my ($Description, $hStruture, $Filter) = @_;#TODO: AD-3112 push additional parameters outside of the wrapper
    LogPayment('PayPalPlus', $Description, $hStruture, undef, $Filter);
    return;
}

1;