#==============================================================================
# package      DE_EPAGES::PayPalPlus::Hooks::Attribute
# state        private
#==============================================================================
package DE_EPAGES::PayPalPlus::Hooks::Attribute;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::ThirdPartyConfig::API::Constants qw ( ATTRIBUTES_CATEGORY_PAYMENT );

use strict;



#========================================================================================
# function     OnGetThirdPartyConfigAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetThirdPartyConfigAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_GetThirdPartyConfigAttributes triggered, register attributes to be
#               configured in PBO
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>AttributesCategory
#                   <li>LanguageID
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetThirdPartyConfigAttributes {
    my ($hParams) = @_;

    return unless $hParams->{'AttributesCategory'} eq ATTRIBUTES_CATEGORY_PAYMENT;

    my $PaymentType  = LoadObjectByPath('/PaymentTypes/PayPalPlus');

    push(@{$hParams->{'Attributes'}}, {
        'SectionID'   => $PaymentType->alias(),
        'SectionName' => $PaymentType->get('NameOrAlias', $hParams->{'LanguageID'}),
        'Attributes' => [
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'Logging'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'LoggingOnboarding'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'ProviderClientId'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'ProviderClientSecret'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'ProviderClientIdSandbox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'ProviderClientSecretSandbox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'ProviderPayPalAccountId'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'MerchantOnboardingURLSandbox'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'MerchantOnboardingURLLive'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'MerchantRestartOnboardingURLLive'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PayPalIPNSettingsLink'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PayPalPlusAccountActivationURL'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PayPalRegisterLink'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PayPalPartnerAttributionId'},
            {'ObjectID' => $PaymentType->id(), 'AttributeAlias' => 'PayPalSupportEmail'},
        ]
    });

    return;
}

1;