#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::Webservice_RestService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Webservice::RestService
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::Webservice_RestService;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::PayPalPlus::API::Webservice::RestService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use HTTP::Request;
use Test::More;

use strict;

sub test {
    my $self = shift;
    $self->test_post();
    $self->test_post_NotOnbordingCall();
    $self->test_post_Error_EmptyRequest();
    $self->test_post_WebserviceError();
    $self->test_post_Error_EmptyResponse();
    $self->test_post_WithHeaders();
    done_testing();
}

sub test_post {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $RestService = $self->_buildRestService();
    my $Url = 'http://Uhr.ar.el';
    my $hHeaders = {'providerClientSecret' => 'MyTestPassword'};
    my $hRequest = {'requestkey' => 'requestvalue'};
    my $IsOnBoardingCall = 1;

    my $WasRerquestFine = $RestService->post($Url, $hHeaders, $hRequest, $IsOnBoardingCall);
    my $hResponse = $RestService->getResponse();

    ok($WasRerquestFine,"$SubTestName - tests if the call was ok");
    my $hExpectedResponse = {'ok'=>'response'};

    is_deeply($hResponse, $hExpectedResponse, "$SubTestName - tests if the response is correct");
    is($RestService->getErrorCode(),undef, "$SubTestName - tests if the error code is undef");
    is($RestService->getErrorMessage(),undef,"$SubTestName - tests if the error message is undef");

    my $Logger = $RestService->{'__ToCheck'}->{'_Logger'};
    my $Params_logPaymentWithoutPassword = GetParametersFromMockifyCall($Logger, 'logPayment');
    my $Param1 = $Params_logPaymentWithoutPassword->[0];
    my $Param2 = $Params_logPaymentWithoutPassword->[1];
    my $Param3 = $Params_logPaymentWithoutPassword->[2];
    is($Param1,'Request params',"$SubTestName - Test log Description");
    my $hExpectedHash = {
            'URL' => 'http://Uhr.ar.el',
            'Headers' => {'providerClientSecret' => 'MyTestPassword'},
            'JSONRequest' => '{requestkey:requestvalue}'
          };
    is_deeply($Param2, $hExpectedHash,"$SubTestName - Test information that is given to log");
    is($Param3,'MyTestPassword',"$SubTestName - test password was given to log to be removed");

    my $aParameters = GetParametersFromMockifyCall($Logger, 'logOnboarding');
    is_deeply($aParameters, [
        {
            'forwarded answer from paypal' => '{ok:response}',
            'epagesj url' => 'https://www.SomeUrl.com/',
            'epagesj request content' => 'Some Content'
        }],
        "$SubTestName - proves that in case of an onbording call the correct values are stored in communication.log"
    );
}
#----------------------------------------------------------------------------------------
sub test_post_NotOnbordingCall {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $RestService = $self->_buildRestService();
    my $Url = 'http://Uhr.ar.el';
    my $hHeaders = {'providerClientSecret' => 'MyTestPassword'};
    my $hRequest = {'requestkey' => 'requestvalue'};

    my $WasRerquestFine = $RestService->post($Url, $hHeaders, $hRequest);
    my $hResponse = $RestService->getResponse();

    ok($WasRerquestFine,"$SubTestName - tests if the call was ok");
    my $hExpectedResponse = {'ok'=>'response'};

    is_deeply($hResponse, $hExpectedResponse, "$SubTestName - tests if the response is correct");
    is($RestService->getErrorCode(),undef, "$SubTestName - tests if the error code is undef");
    is($RestService->getErrorMessage(),undef,"$SubTestName - tests if the error message is undef");

    my $Logger = $RestService->{'__ToCheck'}->{'_Logger'};
    my $Params_logPaymentWithoutPassword = GetParametersFromMockifyCall($Logger, 'logPayment');
    my $Param1 = $Params_logPaymentWithoutPassword->[0];
    my $Param2 = $Params_logPaymentWithoutPassword->[1];
    my $Param3 = $Params_logPaymentWithoutPassword->[2];
    is($Param1,'Request params',"$SubTestName - Test log Description");
    my $hExpectedHash = {
            'URL' => 'http://Uhr.ar.el',
            'Headers' => {'providerClientSecret' => 'MyTestPassword'},
            'JSONRequest' => '{requestkey:requestvalue}'
          };
    is_deeply($Param2, $hExpectedHash,"$SubTestName - Test information that is given to log");
    is($Param3,'MyTestPassword',"$SubTestName - test password was given to log to be removed");

    is(WasCalled($Logger, 'logOnboarding'), 0,"$SubTestName - Prove that the logOnbording was not called when the parameter is not set.");
}
#----------------------------------------------------------------------------------------
sub test_post_Error_EmptyRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $RestService = $self->_buildRestService_for_EmptyRequest();
    my $Url = 'http://Uhr.ar.el';
    my $hHeaders = {'providerClientSecret' => 'MyTestPassword'};
    my $hRequest = '';
    my $IsOnBoardingCall = 1;

    my $WasRerquestFine = $RestService->post($Url, $hHeaders, $hRequest, $IsOnBoardingCall);

    is ($WasRerquestFine, 0, "$SubTestName - tests if the call was not ok");

    is($RestService->getErrorCode(),'WebserviceError', "$SubTestName - tests if the error code is WebserviceError");
    is($RestService->getErrorMessage(), 'JSON parser error', "$SubTestName - tests if the error message is a JSON parser error");
    my $Logger = $RestService->{'__ToCheck'}->{'_Logger'};
    my $aParameters = GetParametersFromMockifyCall($Logger, 'logOnboarding');
    is_deeply($aParameters, [
        {
          'epagesj response' => {
                                  'StatusLine' => 'hash- or arrayref expected'
                                }
        }],
        "$SubTestName - proves that in case of an onbording call the correct values are stored in communication.log if an error happend."
    );
}
#----------------------------------------------------------------------------------------
sub test_post_WebserviceError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $RestService = $self->_buildRestService_for_WebserviceError();
    my $Url = 'http://Uhr.ar.el';
    my $hHeaders = {'providerClientSecret' => 'MyTestPassword'};
    my $hRequest = {'requestkey' => 'requestvalue'};
    my $IsOnBoardingCall = 1;

    my $WasRerquestFine = $RestService->post($Url, $hHeaders, $hRequest, $IsOnBoardingCall);

    is ($WasRerquestFine, 0, "$SubTestName - tests if the call was not ok");

    is($RestService->getErrorCode(),'WebserviceError', "$SubTestName - tests if the error code is WebserviceError");
    like($RestService->getErrorMessage(), qr/a funny WebserviceError/, "$SubTestName - tests if the error message is a webservice error");
    my $Logger = $RestService->{'__ToCheck'}->{'_Logger'};
    my $aParameters = GetParametersFromMockifyCall($Logger, 'logOnboarding');
    is_deeply($aParameters, [ {
          'epagesj response' => { 'StatusLine' => 'a funny WebserviceError' }
        }],
        "$SubTestName - proves that in case of an onbording call the correct values are stored in communication.log if an error happend."
    );
}
#----------------------------------------------------------------------------------------
sub test_post_Error_EmptyResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $RestService = $self->_buildRestService_for_EmptyResponse();
    my $Url = 'http://Uhr.ar.el';
    my $hHeaders = {'providerClientSecret' => 'MyTestPassword'};
    my $hRequest = {'requestkey' => 'requestvalue'};
    my $IsOnBoardingCall = 1;

    my $WasRerquestFine = $RestService->post($Url, $hHeaders, $hRequest, $IsOnBoardingCall);

    is ($WasRerquestFine, 0, "$SubTestName - tests if the call was not ok");

    is($RestService->getErrorCode(),'WebserviceError', "$SubTestName - tests if the error code is WebserviceError");
    is($RestService->getErrorMessage(), 'JSON parser error', "$SubTestName - tests if the error message is a JSON parser error");
    my $Logger = $RestService->{'__ToCheck'}->{'_Logger'};
    my $aParameters = GetParametersFromMockifyCall($Logger, 'logOnboarding');
    is_deeply($aParameters, [ {
          'epagesj response' => { 'StatusLine' => 'malformed JSON string' }
        }],
        "$SubTestName - proves that in case of an onbording call the correct values are stored in communication.log if an error happend."
    );
}
#----------------------------------------------------------------------------------------
sub test_post_WithHeaders {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $RestService = $self->_buildRestService_WithHeaders();
    my $Url = 'http://Uhr.ar.el';
    my $hHeaders = {'header1' => 'headervalue', 'providerClientSecret' => 'MyTestPassword'};
    my $hRequest = {'requestkey' => 'requestvalue'};
    my $IsOnBoardingCall = 1;

    my $WasRerquestFine = $RestService->post($Url, $hHeaders, $hRequest, $IsOnBoardingCall);
    my $hResponse = $RestService->getResponse();

    ok($WasRerquestFine,"$SubTestName - tests if the call was ok");
    my $hExpectedResponse = {'ok'=>'response'};

    is_deeply($hResponse, $hExpectedResponse, "$SubTestName - tests if the response is correct");
    is($RestService->getErrorCode(),undef, "$SubTestName - tests if the error code is undef");
    is($RestService->getErrorMessage(),undef,"$SubTestName - tests if the error message is undef");

    my $Logger = $RestService->{'__ToCheck'}->{'_Logger'};
    my $Params_logPaymentWithoutPassword = GetParametersFromMockifyCall($Logger, 'logPayment');
    my $Param1 = $Params_logPaymentWithoutPassword->[0];
    my $Param2 = $Params_logPaymentWithoutPassword->[1];
    my $Param3 = $Params_logPaymentWithoutPassword->[2];
    is($Param1,'Request params',"$SubTestName - Test log Description");
    my $hExpectedHash = {
            'URL' => 'http://Uhr.ar.el',
            'Headers' => {'header1' => 'headervalue', 'providerClientSecret' => 'MyTestPassword'},
            'JSONRequest' => '{requestkey:requestvalue}'
          };
    is_deeply($Param2, $hExpectedHash,"$SubTestName - Test information that is given to log");
    is($Param3,'MyTestPassword',"$SubTestName - test password was given to log to be removed");

    my $aParameters = GetParametersFromMockifyCall($Logger, 'logOnboarding');
    is_deeply($aParameters, [
        {
            'forwarded answer from paypal' => '{ok:response}',
            'epagesj url' => 'https://www.SomeUrl.com/',
            'epagesj request content' => 'Some Content'
        }],
        "$SubTestName - proves that in case of an onbording call the correct values are stored in communication.log"
    );
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildRestService {
    my $self = shift;
    my $RestClient = $self->_createRestClient();
    return $self->_createRestService($RestClient);
}
#----------------------------------------------------------------------------------------
sub _buildRestService_for_EmptyRequest {
    my $self = shift;
    my $RestClient = $self->_buildRestClient_for_EmptyRequest();
    return $self->_createRestService($RestClient);
}
#----------------------------------------------------------------------------------------
sub _buildRestService_for_EmptyResponse {
    my $self = shift;
    my $RestClient = $self->_buildRestClient_for_EmptyResponse();
    return $self->_createRestService($RestClient);
}
#----------------------------------------------------------------------------------------
sub _buildRestService_for_WebserviceError {
    my $self = shift;
    my $RestClient = $self->_buildRestClient_for_WebserviceError();
    return $self->_createRestService($RestClient);
}
#----------------------------------------------------------------------------------------
sub _buildRestService_WithHeaders {
    my $self = shift;
    my $RestClient = $self->_buildRestClient_WithHeaders();
    return $self->_createRestService($RestClient);
}

#----------------------------------------------------------------------------------------
sub _createRestService {
    my $self = shift;
    my ($RestClient) = @_;
    my $Shop = $self->_createShop();
    my $aParameterList = [$Shop];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::RestService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createRestClient',$RestClient,[{'object'=>'DE_EPAGES::Shop::API::Object::Shop'},{'hashref'=>{}}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('_LogError',1,[{'object'=>'DE_EPAGES::Core::API::Error'}]);

    my $Logger = $self->_createLogger();
    $Mockify->addMockWithReturnValue('_createLogger', $Logger);

    my $RestService = $Mockify->getMockObject();
    $RestService->{'__ToCheck'}->{'_Logger'} = $Logger;
    return $RestService;
}
#----------------------------------------------------------------------------------------
sub _createLogger {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Logger', [] );

    $Mockify->addMockWithReturnValueAndParameterCheck('logPayment', undef, ['string', 'hashref', 'string']);
    $Mockify->mock('logOnboarding', undef, ['hashref']);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRestClient {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('encodeRequest','{requestkey:requestvalue}', [{'hashref' => {'requestkey' => 'requestvalue'}}]);
    my $HttpClient = $self->_createHTTPClient();
    $Mockify->addMockWithReturnValue('client',$HttpClient);
    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse',{'ok'=>'response'},[{'string' => '{ok:response}'}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildRestClient_for_EmptyRequest {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMock('encodeRequest', sub{
        $self->_throwError('NativeDie','hash- or arrayref expected');
    });

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildRestClient_for_EmptyResponse {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('encodeRequest','{requestkey:requestvalue}', [{'hashref' => {'requestkey' => 'requestvalue'}}]);
    my $HttpClient = $self->_createHTTPClient();
    $Mockify->addMockWithReturnValue('client',$HttpClient);
    $Mockify->addMock('decodeResponse', sub{
        $self->_throwError('NativeDie','malformed JSON string');
    });

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildRestClient_for_WebserviceError {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('encodeRequest','{requestkey:requestvalue}', [{'hashref' => {'requestkey' => 'requestvalue'}}]);
    my $HttpClient = $self->_createHTTPClient_for_WerbserviceError();
    $Mockify->addMockWithReturnValue('client',$HttpClient);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildRestClient_WithHeaders {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::RestClient',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('encodeRequest','{requestkey:requestvalue}', [{'hashref' => {'requestkey' => 'requestvalue'}}]);
    my $HttpClient = $self->_createHTTPClient_WithHeaders();
    $Mockify->addMockWithReturnValue('client',$HttpClient);
    $Mockify->addMockWithReturnValueAndParameterCheck('decodeResponse',{'ok'=>'response'},[{'string' => '{ok:response}'}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createHTTPClient {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::HTTPClient',
                   $aParameterList
              );
    my $HttpResponse = $self->_createHTTPResponse();
    $Mockify->addMockWithReturnValueAndParameterCheck('post',$HttpResponse,
    [
        {'string' => 'http://Uhr.ar.el'},
        {'hashref' => {'providerClientSecret' => 'MyTestPassword'}},
        {'string' => '{requestkey:requestvalue}'}
    ]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createHTTPClient_WithHeaders {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::HTTPClient',
                   $aParameterList
              );
    my $HttpResponse = $self->_createHTTPResponse();
    $Mockify->addMockWithReturnValueAndParameterCheck('post',$HttpResponse,
    [
        {'string' => 'http://Uhr.ar.el'},
        {'hashref' => {'header1'=>'headervalue', 'providerClientSecret' => 'MyTestPassword'}},
        {'string' => '{requestkey:requestvalue}'}]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createHTTPClient_for_WerbserviceError {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::EPagesJ::API::Object::HTTPClient',
                   $aParameterList
              );
    $Mockify->addMock('post', sub {
        $self->_throwError('REQUEST_FAILED','a funny WebserviceError');
    });
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createHTTPResponse {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'HTTP::Response',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('content','{ok:response}');
    $Mockify->mock('request', $self->_createRequest(), []);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRequest {
    my $self = shift;
    my $Request = HTTP::Request->new(GET => 'https://www.SomeUrl.com/');
    $Request->add_content('Some Content');
    return $Request;
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _throwError {
    my $self = shift;
    my ( $Code, $Message ) = @_;

    my $Error = DE_EPAGES::Core::API::Error->new(
        'Code' => $Code,
        'Message' => $Message,
        'Vars' => {
            'StatusLine' => $Message,
        }
    );
    Error($Error);

    return;
}

__PACKAGE__->runScript();