#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler
# base         DE_EPAGES::Trigger::API::Object::BatchProcessHandler
# state        public
#----------------------------------------------------------------------------------------
# description  BatchProcessHandler for Category CRUD Events
#========================================================================================
package DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler;
use base DE_EPAGES::Trigger::API::Object::BatchProcessHandler;

use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw ( RunIfRemoteSearchInitialized );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $hOptions );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $hOptions |<ul><li>BatchSize - integer > 1 (optional)</li></ul> | ref.hash
# return       $self | self | DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler
#========================================================================================
sub new {
    my $class = shift;
    my ($hOptions) = @_;
    return $class->SUPER::new($hOptions);
}
#----------------------------------------------------------------------------------------
sub processItemList {
    my $self = shift;
    my $aShops = $self->items();

    return unless ($aShops && ref($aShops) eq 'ARRAY' && $aShops->[0]);

    my $FirstShop = $aShops->[0];# since we do always an reindex of the full shop we don't need to take care about the single Items.
    # If we somehow do later an direct category update. Keep an eye on the products which sometimes change too. ( e.g. on delete category)
    $self->_RunIfRemoteSearchInitialized(
        $FirstShop,
        sub {
            my ($Search) = @_;
            $Search->initialize();
        }
    );

    $self->cleanupItems();
    return;
}

#----------------------------------------------------------------------------------------
sub addItem {
    my $self = shift;
    my ($Shop) = @_;
    TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    push(@{ $self->items() }, $Shop);

    return;
}
#----------------------------------------------------------------------------------------
sub _RunIfRemoteSearchInitialized {
    my $self = shift;
    my ($Shop, $cSub) = @_;
    RunIfRemoteSearchInitialized($Shop, $cSub);
}
1;
