#========================================================================================
# package      DE_EPAGES::RemoteSearch::Hooks::CategoryEvents
# state        public
#----------------------------------------------------------------------------------------
# description  Handler for category-related hooks
#========================================================================================
package DE_EPAGES::RemoteSearch::Hooks::CategoryEvents;

use DE_EPAGES::Trigger::API::BatchProcessor qw (
    HandleItemInBatchModeOnly
    RegisterHandler
    IsCalledInBatchMode
);

use DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler;

use strict;

sub OnRegisterCreateCategoriesHandler {
    RegisterHandler('CreateCategories', DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler->new());
    return;
};

sub OnRegisterUpdateCategoriesHandler {
    RegisterHandler('UpdateCategories', DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler->new());
    return;
};

sub OnRegisterDeleteCategoriesHandler {
    RegisterHandler('DeleteCategories', DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler->new());
    return;
};

sub OnInsertCategory {
    my ($hParams) = @_;
    # csv-import triggers create & update, and if the context allows both we will only do the update
    if(IsCalledInBatchMode('UpdateCategories')){
        _TriggerBatchEvent($hParams, 'UpdateCategories');
    }else{
        _TriggerBatchEvent($hParams, 'CreateCategories');
    }
    return;
}

sub OnAfterUpdateCategory {
    my ($hParams) = @_;
    _TriggerBatchEvent($hParams, 'UpdateCategories');
    return;
}

sub OnDeleteCategory {
    my ($hParams) = @_;
    _TriggerBatchEvent($hParams, 'DeleteCategories');
    return;
}

sub _TriggerBatchEvent {
    my ($hParams, $BatchTaskName) = @_;

    my $Category = $hParams->{'Object'};
    return unless ($Category and $Category->instanceOf('Category'));
    my $Shop = $Category->getSite();

    HandleItemInBatchModeOnly(
        $BatchTaskName,
        'DE_EPAGES::RemoteSearch::API::Object::CategoryBatchProcessHandler',
        $Shop
    );
    return;
}

1;