#========================================================================================
# package      DE_EPAGES::RemoteSearch::Hooks::CategoryProductMap
# state        private
# description  Handles CategoryProductMap updates (Postion/PositionCategory changes). 
#               Supports batch context
#               (IsCalledInBatchMode). See <package DE_EPAGES::Trigger::API::BatchProcessor>
#========================================================================================
package DE_EPAGES::RemoteSearch::Hooks::CategoryProductMap;

use strict;

use DE_EPAGES::RemoteSearch::API::AttributeModifiedCheck qw ( RunIfModified );
use DE_EPAGES::RemoteSearch::API::Batch qw ( BatchUpdateProduct BatchUpdateCategoryProductMap);
use DE_EPAGES::RemoteSearch::API::Object::ProductCategoryMappingBatchProcessHandler;

use DE_EPAGES::Trigger::API::BatchProcessor qw (
    RegisterHandler
);

sub OnRegisterUpdateCategoriesHandler {
    RegisterHandler('CSVCatalogImport', DE_EPAGES::RemoteSearch::API::Object::ProductCategoryMappingBatchProcessHandler->new());
    return;
};

#========================================================================================
# function     OnBeforeUpdateCategoryProductMap
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateCategoryProductMap( $hInfo );
#----------------------------------------------------------------------------------------
# description  Communicates ProductCategoryMap changes to remote search
#----------------------------------------------------------------------------------------
# input        $hInfo | objects
#               <ul>
#                   <li>Object - object - object</li>
#                   <li>Values - values about to be changed - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateCategoryProductMap {
    my ($hInfo) = @_;

    my $CategoryProductMap = $hInfo->{'Object'};
    my $hValues = $hInfo->{'Values'};
    my $AssignedProduct = $CategoryProductMap->get('Product');

    #The position is used to order the products in the Solr search when using: 'sorting by relevance'(EPG-26816)
    my @NON_LOCALIZED = qw ( Position PositionCategory );
    my @NON_LOC_TYPES = qw ( Integer Integer );

    RunIfModified(
        $CategoryProductMap,
        \@NON_LOCALIZED,
        $hValues, undef,
        \@NON_LOC_TYPES,
        sub {
           # this will update the products if the hook was not triggert in context of CSVCatalogImport
           BatchUpdateProduct($AssignedProduct);
        }
    );
    # this will reindex the Shop, only if this hook was trigged in the context of an CSVCatalogImport
    BatchUpdateCategoryProductMap($AssignedProduct);

    return;
}



1;