#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::unit::ProductCategoryMappingBatchProcessHandler
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::RemoteSearch::API::Object::ProductCategoryMappingBatchProcessHandler
#========================================================================================
package DE_EPAGES::RemoteSearch::t::unit::ProductCategoryMappingBatchProcessHandler;
use base DE_EPAGES::Test::API::UnitTest::Test;
use Test::More;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object Function );
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall WasCalled);
use strict;

sub test {
    my $self = shift;


    $self->test_addItem();
    $self->test_processItemList();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_addItem {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $SUT = $self->_createSUT();
    my $Shop = $self->_createShop();

    $SUT->addItem($Shop);

    is($SUT->countItems(), 1, "$SubTestName - Prove the Item count of the internal stack.");
    is($SUT->batchSize(), 1000, "$SubTestName - Prove the BatchSize which were initiated in the constructor.");
    isa_ok($SUT->items()->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - check first element of the stack.");
    $SUT->cleanupItems();
    is($SUT->countItems(), 0, "$SubTestName - Prove the Item count of the internal stack after clean up.");
}
#----------------------------------------------------------------------------------------
sub test_processItemList {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $SUT = $self->_createSUT();
    my $Shop = $self->_createShop();
    my $SearchIndex = $self->_createSearchIndex();

    $SUT->addItem($Shop);

    is($SUT->countItems(), 1, "$SubTestName - Prove the Item count of the internal stack.");
    $SUT->processItemList();
    is($SUT->countItems(), 0, "$SubTestName - Prove the Item count of the internal stack after clean up.");

    my $aCallParameter = GetParametersFromMockifyCall($SUT,'_RunIfRemoteSearchInitialized');
    isa_ok($aCallParameter->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - Check first parameter.");
    $aCallParameter->[1]->($SearchIndex);
    ok(WasCalled($SearchIndex, 'initialize'), "$SubTestName - Check the second parameter and prove that the anonymous function was called.");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $aParameterList = [{'BatchSize' => 1000}];
    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::RemoteSearch::API::Object::ProductCategoryMappingBatchProcessHandler',
                   $aParameterList
              );
    $Mockify->mock('_RunIfRemoteSearchInitialized')
            ->when( Object(), Function() )
            ->thenReturnUndef();

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSearchIndex {
    my $self = shift;

    my $hAttributes = {};
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::RemoteSearch::API::Object::SearchIndex',
        $hAttributes
    );
    $Mockify->mock('initialize', undef, []);

    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();