#========================================================================================
# package      DE_EPAGES::Spreedly::UI::Shop
# base         DE_EPAGES::Order::UI::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  UI Controller for shop
#========================================================================================
package DE_EPAGES::Spreedly::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::Core::API::PerlTools qw ( AddDuration );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Spreedly::API::Constants qw ( INTERNAL_CARTRIDGE_NAME );

use strict;


#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName($Servlet);
#----------------------------------------------------------------------------------------
# description  Name of PaymentType for search/view transactions.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {
    my $self = shift;
    my ($Servlet) = @_;

    if (defined $Servlet) {
        my $Params = $Servlet->params();

        my $PaymentName = $Params->{'ViewAction'};
        $PaymentName =~ s/.*Search(\w+)Transactions/$1/g;

        return $PaymentName;
    }
    return INTERNAL_CARTRIDGE_NAME;
}



#========================================================================================
# function     PaymentBacklinkSpreedly
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentBacklinkSpreedly( $Servlet );
#----------------------------------------------------------------------------------------
# description  TODO: Remove when not used!
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub PaymentBacklinkSpreedly {
    my $self = shift;
    my ($Servlet) = @_;

    # compare with other cartridges to find out what is needed here:

    # - BlockMonitor
    # - Log
    # - Reset: locale, languageId, currencyId
    # - FormError
    # - $Servlet->registerSession($CustomerOrder->get('User'),0 ); ?!
    # - doBasket2Order?
    # - setTransStatus and -Time

    return;
}



#========================================================================================
# function     PaymentCallbackSpreedly
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentCallbackSpreedly( $Servlet );
#----------------------------------------------------------------------------------------
# description  Server-server callback from payment provider #TODO: Remove when not used!
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub PaymentCallbackSpreedly {
    my $self = shift;
    my ($Servlet) = @_;

    # compare with other cartridges to find out what is needed here:

    # - Log
    # - FormError
    # - $Servlet->registerSession($CustomerOrder->get('User'),0 ); ?!
    # - doBasket2Order?
    # - setTransStatus and -Time

    $Servlet->params('ContentType', 'text/plain');
    $Servlet->params('ExitAfterEvent', 1);
    $Servlet->writeContentOutput('OK');

    return;
}

#========================================================================================
# function     SearchTransactions
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SearchTransactions( $Servlet );
#----------------------------------------------------------------------------------------
# description  Searches for transactions of the current payment type.
#               The TLE variable #Pager contains the search results.
#               See also <function TransactionPaymentName>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub SearchTransactions {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $SearchActionName = 'Search' . $self->TransactionPaymentName($Servlet) . 'Transactions';
    my $SearchForm = $Servlet->form->form($Shop, $SearchActionName);

    my $SearchTemplateName = $self->TransactionPaymentName($Servlet) . 'Transactions';
    $SearchForm->{'Search'} = $SearchTemplateName;

    #--- default values
    if (!defined $Servlet->params('OrderBy')) {
        $Servlet->params('OrderBy', 'TransTime');
        $Servlet->params('OrderDesc', 1);
    }

    $Servlet->vars('ViewSearchAction', 'MBO-' . $SearchActionName);

    return $self->_ExecuteTransactionSearch($Servlet, $SearchForm);
}

#========================================================================================
# function     _ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# return       $hPagerVars | pager variables that can be used inside a template (cf. $Pager-><function page>) | ref.hash
#========================================================================================
#========================================================================================
# function     ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub _ExecuteTransactionSearch {
    my $self = shift;
    my ($Servlet, $SearchForm) = @_;

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    $SearchForm->{'Search'} = 'Transactions' unless exists $SearchForm->{'Search'};
    my $Shop = $Servlet->object;
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('LineItemPayment' . $self->TransactionPaymentName()), $SearchForm->{'Search'});

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'IsVisible'  => 1,
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    my $hSearchOptions = $self->SearchOptions();
    $Options{'Parameters'}{$_} = $hSearchOptions->{$_} foreach keys %$hSearchOptions;

    delete $SearchForm->{'Search'};
    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        } elsif ($Parameter eq 'TransTimeTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);

        }else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

        # EPG-12969: use original input values in pager URL
        if( $Parameter =~ /^TransTime(From|To)$/ ) {
            $Options{'UriParameters'}{$Parameter} = $Servlet->form->value($Parameter);
        }
        else {
            $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}
1;