#========================================================================================
# package      DE_EPAGES::TLE::API::FormatHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This handler supports the following tle function:
#               <ul>
#                 <li>FORMAT - execute the formatter.
#               </ul>
# example      #FUNCTION("FORMAT", #Value, #Format, #Name)
# example      #FUNCTION("FORMAT", #Now, "datetime;small", "Now") same as #Now[datetime;small] but variable.
#========================================================================================
package DE_EPAGES::TLE::API::FormatHandler;

use strict;

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::TLE::API::FormatHandler;
# example      my $SpaceFormatter  = new DE_EPAGES::TLE::API::FormatHandler;
#----------------------------------------------------------------------------------------
# description  Creates a new handler.
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { return bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  This function registers this handle on the processor.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    $Processor->registerHandler('FunctionHandler', $self, 'FORMAT');
    return 1;
}

#========================================================================================
# function     FORMAT
# state        private
#----------------------------------------------------------------------------------------
# syntax       #FUNCTION("FORMAT", #Value, #Format)
# syntax       #FUNCTION("FORMAT", #Value, #Format, #Name, ...)
# example      #FUNCTION("FORMAT", #CreationDate, "datetime")
# example      #FUNCTION("FORMAT", #CreationDate, "datetime", "CreationDate", "en_GB", "%a, %d %b %Y %H:%M:%S %z")
#----------------------------------------------------------------------------------------
# description  Returns formatted value.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $Value | value | *
# input        $Format | format | string
# input        $Name | name of tle (optional) | string
#========================================================================================
sub FORMAT {
    my $self = shift;
    my ($Processor, $raParams) = @_;

    my ($Value, $Format, $Name, @Parameter) = @$raParams;
    return undef unless defined $Value;
    $Name = 'FormatFunction' unless defined $Name;
    return $Processor->callFormatHandler($Value, $Format, $Name, \@Parameter);
}

1;
